/** 
	 * FullMouseMode.java
	 * Autor: Alberto Población Navarro
	 * Añadido a:
	 		* Archivo original: http://code.google.com/p/android-vnc-viewer/source/browse/branches/antlersoft/androidVNC/src/android/androidVNC/VncCanvasActivity.java
	 		* Release: 39
	 */
package android.androidVNC;

import java.util.Date;

import android.content.Context;
import android.os.CountDownTimer;
import android.view.Display;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.widget.Toast;

	/**
	 * Modo para un control global del raton y el teclado remotos
	 * @author Alberto Población Navarro
	 *
	 */
	class FullMouseMode implements AbstractInputHandler {
	
		public VncCanvas vncCanvas;
		public VncCanvasActivity vncCanvasActivity;
		private Sensores acelerometro;
		/**
		 *  Modo para un control global del ratón y el teclado remotos
		 *  @param vnccanvas instancia de la clase VncCanvas que se está utilizando, es la imagen que se dibuja
		 *  @param vnccanvasactivity instacia de la clase VncCanvasActivity, la encargada de intercambiar informacion entre el rfb protocol y el vnccanvas
		 */
		FullMouseMode(VncCanvas vnccanvas, VncCanvasActivity vnccanvasactivity)
		{
			vncCanvas = vnccanvas;
			vncCanvasActivity = vnccanvasactivity;
			acelerometro = new Sensores(vncCanvasActivity, vncCanvas);
		}
		

		
		/* (non-Javadoc)
		 * @see android.androidVNC.AbstractInputHandler#onKeyDown(int, android.view.KeyEvent)
		 */
		@Override
		public boolean onKeyDown(int keyCode, KeyEvent evt) {
			if(keyCode == KeyEvent.KEYCODE_DPAD_CENTER){
				KeyEvent camera = new KeyEvent(KeyEvent.ACTION_DOWN,KeyEvent.KEYCODE_CAMERA);
				vncCanvas.processLocalKeyEvent(KeyEvent.KEYCODE_CAMERA, camera);
				vncCanvas.processPointerEvent(MotionEvent.obtain(1, System.currentTimeMillis(), MotionEvent.ACTION_DOWN, vncCanvas.mouseX,vncCanvas.mouseY, 0), false);			
				}
			else{
				vncCanvasActivity.defaultKeyDownHandler( keyCode, evt);
			}
				
				return true;
		}
		
		/* (non-Javadoc)
		 * @see android.androidVNC.AbstractInputHandler#onKeyUp(int, android.view.KeyEvent)
		 */
		@Override
		public boolean onKeyUp(int keyCode, KeyEvent evt) {
						
			if(keyCode == KeyEvent.KEYCODE_DPAD_CENTER){
				vncCanvas.processPointerEvent(MotionEvent.obtain(1, System.currentTimeMillis(), MotionEvent.ACTION_UP, vncCanvas.mouseX,vncCanvas.mouseY, 0), false);
				KeyEvent camera = new KeyEvent(KeyEvent.ACTION_UP,KeyEvent.KEYCODE_CAMERA);
				vncCanvas.processLocalKeyEvent(KeyEvent.KEYCODE_CAMERA, camera);		
				}
			else{
				vncCanvasActivity.defaultKeyDownHandler( keyCode, evt);
			}
				return true;
		}
	
		/* (non-Javadoc)
		 * @see android.androidVNC.AbstractInputHandler#onTouchEvent(android.view.MotionEvent)
		 */
		@Override
		public boolean onTouchEvent(MotionEvent event) {
			return fullmousetouch(event);
		}
		/* (non-Javadoc)
		 * @see android.androidVNC.AbstractInputHandler#onTrackballEvent(android.view.MotionEvent)
		 */
		@Override
		public boolean onTrackballEvent(MotionEvent evt) {
			return false;
		}
		/* (non-Javadoc)
		 * @see android.androidVNC.AbstractInputHandler#handlerDescription()
		 */
		@Override
		public CharSequence getHandlerDescription() {
			return vncCanvasActivity.getResources().getText(R.string.input_mode_full_mouse);
		}

        /* (non-Javadoc)
         * @see android.androidVNC.AbstractInputHandler#getName()
         */
        @Override
        public String getName() {
                return "FULL MOUSE";
        }
		
	
	private Date down; //Momento de la pulsacion
	private Date up; // Momento en el que se suelta la pulsacion
	private final int click = 100; //Tiempo en milisegundos para saber que se trata de un click
	private final int drag = 500; //Tiempo en milisegundos para saber si hay que arrastrar en vez de mover
		
	CountDownTimer timer; //Temporizador para saber si se trata de un click o para arrastrar
	
	/**
	 * Segun el parametro event, realiza una acción que se será enviada al servidor como acción de ratón.
	 * Depende de la acción podemos mandar un movimiento, un click o la acción de arrastrar
	 * @param event evento tactil que sera tratado por la funcion
	 * @return 
	 * @author Alberto Poblacion Navarro
	 */
	boolean fullmousetouch( MotionEvent event) 
	{
		switch (event.getAction()) {
		case MotionEvent.ACTION_DOWN:
			down=new Date();//Momento de la pulsacion
			final MotionEvent evento = MotionEvent.obtain(event);
			final Context contex_=vncCanvasActivity;
			
			
			//Si es una pulsacion larga: modo de arrastre. 
			timer = new CountDownTimer(drag, 10){

			     public void onTick(long millisUntilFinished) {
			     }
			     				 
			     public void onFinish() {
			   
			    	 Toast.makeText(vncCanvasActivity, "Dragging Mode", Toast.LENGTH_SHORT).show();
					 evento.setAction(MotionEvent.ACTION_DOWN);
					 // Adjust coordinates for Android notification bar.
					 evento.offsetLocation(0, -1f * vncCanvas.getTop());
					 // Adjust coordinates for panning position.
					 evento.offsetLocation(vncCanvasActivity.absoluteXPosition, vncCanvasActivity.absoluteYPosition);
					 vncCanvas.processPointerEvent(evento, true);
				}
			  }.start();
		
			break;
		case MotionEvent.ACTION_MOVE:
			
			Move(event);
			//Cancelas el timeout para que no entre en el modo de arrastre
			timer.cancel();
					
			break;
		case MotionEvent.ACTION_UP:
			up=new Date(); //Para conocer la duracion de la pulsacion
			timer.cancel();
			//Si es una pulsacion rapida: click de raton
			if (up.getTime() - down.getTime() < click)
			{
				//Mandas el gesto del click complento, cuando pulsas y cuando sueltas la tecla
				event.setAction(MotionEvent.ACTION_DOWN);
				// Adjust coordinates for Android notification bar.
				event.offsetLocation(0, -1f * vncCanvas.getTop());
				// Adjust coordinates for panning position.
				event.offsetLocation(vncCanvasActivity.absoluteXPosition, vncCanvasActivity.absoluteYPosition);
				//Mandas el click completo: pulsas y sueltas
				vncCanvas.processPointerEvent(event, true);
				event.setAction(MotionEvent.ACTION_UP);
				vncCanvas.processPointerEvent(event, true);
				break;
			}
			//Si no ha sido un click del raton mandas que se levantado la tecla simplemente
			event.setAction(MotionEvent.ACTION_UP);
			// Adjust coordinates for Android notification bar.
			event.offsetLocation(0, -1f * vncCanvas.getTop());
			// Adjust coordinates for panning position.
			event.offsetLocation(vncCanvasActivity.absoluteXPosition, vncCanvasActivity.absoluteYPosition);
			vncCanvas.processPointerEvent(event, true);
			break;
			
		}
		return true;		
	}
	
	final int border = 20; //Distancia entre el cursor y el borde para que se desplace la pantalla
	/**
	 * Mueve la parte de escritorio en la que nos encontramos si estamos a una distancia menor de X de un extremo
	 * @param event
	 * @author Alberto Poblacion Navarro
	 */
	private void Move(MotionEvent event) {
		//Obtienes las cordenadas donde tienes el cursor
		
		float curX = event.getX();
		float curY = event.getY();
		int scrollX =0;
		int scrollY =0;
		
		//Si pulsamos el raton a menos distancia de "border" de uno de los extremos, configuramos la posicion que
		//debemos mover la imagen
		Display display=vncCanvasActivity.getWindowManager().getDefaultDisplay();
		if (curX  > (display.getWidth()-border)) scrollX +=border;
		if (curX  < border) scrollX -=border;
		if (curY  > (display.getHeight()-border)) scrollY +=border;
		if (curY  < border) scrollY -=border;
			
			
			
		vncCanvasActivity.absoluteXPosition += scrollX;
		vncCanvasActivity.absoluteYPosition += scrollY;
		//Comprobamos los margenes superior e izquierdo
		if(vncCanvasActivity.absoluteXPosition < 0){
			scrollX=0;
			vncCanvasActivity.absoluteXPosition=0;
		}
		if(vncCanvasActivity.absoluteYPosition < 0){
			scrollY=0;
			vncCanvasActivity.absoluteYPosition=0;
		}
		//Comprobamos los márgenes derecho y inferior 
		if (vncCanvasActivity.absoluteXPosition + vncCanvas.getWidth()> vncCanvas.getImageWidth()){
			scrollX=0;
			vncCanvasActivity.absoluteXPosition-=border;
		}
		if (vncCanvasActivity.absoluteYPosition + vncCanvas.getHeight() > vncCanvas.getImageHeight()){
			scrollY=0;
			vncCanvasActivity.absoluteYPosition-=border;
		}
		//Movemos la posicion del escritorio
		vncCanvas.scrollBy(scrollX, scrollY);
		
		event.offsetLocation(0, -1f * vncCanvas.getTop());
		event.offsetLocation(vncCanvasActivity.absoluteXPosition,vncCanvasActivity.absoluteYPosition);
		vncCanvas.processPointerEvent(event, false);
		
	}
}